/*
Copyright (C) 2015 Alexey Karyakin <alexey.karyakin@uwaterloo.ca>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the "Software"), to deal in the Software without
restriction, including without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#include "Reader.h"

#include <errno.h>
#include <unistd.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>

Reader::Reader(int fd, unsigned ring_bits, PageStore *page_store):
	m_fd(fd),
	m_allocator(ring_bits, page_store),
	m_pos((char *)m_allocator.resolve(m_allocator.head())),
	m_end((char *)m_allocator.resolve(m_allocator.head())),
	m_offset(0)
{
}

void *Reader::read_slow(uint32_t sz)
{
	assert(m_pos + sz > m_end);
	size_t to_read = m_pos + sz - m_end;
	size_t available = m_allocator.diff_gross(m_allocator.head(), m_allocator.offset(m_end)); // NOTE: no overhead
	if (available < to_read)
	{
		to_read = (to_read  - available + READ_SZ - 1) & ~(READ_SZ - 1);
		char *next_block = (char *)m_allocator.resolve(m_allocator.allocate(to_read));
		if (next_block < m_end)
		{
			if (to_read < sz)
				m_allocator.allocate(READ_SZ);
			memcpy(next_block, m_pos, m_end - m_pos);
			m_end = next_block + (m_end - m_pos);
			m_pos = next_block;
		}
	}
	char *limit = (char *)m_allocator.resolve(m_allocator.head());
	while (m_pos + sz > m_end)
	{
		auto n = ::read(m_fd, m_end, limit - m_end);
		if (n == 0)
		{
			if (m_pos + sz > m_end)
				return NULL;
			break;
		}
		if (n < 0)
		{
			if (errno == EINTR)
				continue;
			abort();
		}
		m_end += n;
		m_offset += n;
	}
	void *p = m_pos;
	m_pos += sz;
	return p;
}

void Reader::release(Bookmark bookmark)
{
	m_allocator.collect(bookmark.ofs);
}
