/*
Copyright (C) 2015 Alexey Karyakin <alexey.karyakin@uwaterloo.ca>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the "Software"), to deal in the Software without
restriction, including without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef Reader_H
#define Reader_H

#include "likely.h"
#include "LinearAllocator.h"

#include <stdio.h>

class Reader
{
public:

	enum { ALLOC_BITS = 21, READ_BITS = 20 };
	enum : size_t { READ_SZ = size_t(1) << READ_BITS };

	static_assert(READ_BITS <= ALLOC_BITS, "");

	struct Bookmark
	{
		size_t ofs;
	};

	Reader(int fd, unsigned ring_bits, PageStore *page_store);

	template <typename T>
	const T *read(size_t sz = sizeof(T))
	{
		return static_cast<const T *>(read(sz));
	}
	void *read(uint32_t sz)
	{
		char *next = m_pos + sz;
		if (likely(next <= m_end))
		{
			void *p = m_pos;
			m_pos = next;
			return p;
		}
		return read_slow(sz);
	}
	uint64_t count() const
	{
		return m_offset - (m_end - m_pos);
	}
	Bookmark bookmark()
	{
		return Bookmark{m_allocator.offset(m_pos)};
	}
	void release(Bookmark);

private:

	int m_fd;
	LinearAllocator<ALLOC_BITS> m_allocator;
	char *m_pos;
	char *m_end;

	uint64_t m_offset;

	void *read_slow(uint32_t sz);
};

#endif

