/*
Copyright (C) 2015 Alexey Karyakin <alexey.karyakin@uwaterloo.ca>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the "Software"), to deal in the Software without
restriction, including without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef AGGR_H
#define AGGR_H

#include "def.h"
#include "util.h"
#include <assert.h>

enum AggrOp { MIN, MAX };

template <AggrOp>
struct AggrIntImpl;

template <>
struct AggrIntImpl<MIN>
{
	static void aggregate(uint64_t &a, uint64_t v)
	{
		if (unlikely(v < a))
			a = v;
	}
	static value_t init() { return std::numeric_limits<value_t>::max(); }
};

template <>
struct AggrIntImpl<MAX>
{
	static void aggregate(uint64_t &a, uint64_t v)
	{
		if (unlikely(v > a))
			a = v;
	}
	static value_t init() { return std::numeric_limits<value_t>::min(); }
};

template <AggrOp Op>
struct AggrInt
{
	enum { BITS = 0, N = 1 << BITS };
	static void aggregate(value_t *a, const value_t *v)
	{
		AggrIntImpl<Op>::aggregate(*a, *v);
	}
	static value_t init() { return AggrIntImpl<Op>::init(); }
};

template <typename Impl, unsigned BlockingBits>
void aggregate(unsigned n, value_t *a, const value_t *v)
{
	static_assert(BlockingBits >= Impl::BITS, "");
	assert(alignOffsetBits(n, BlockingBits) == 0);
	assert(n != 0);
	do
	{
		for (unsigned i = 0; i != 1 << (BlockingBits - Impl::BITS); i++)
			Impl::aggregate(a + i * Impl::N, v + i * Impl::N);
		a += 1 << BlockingBits;
		v += 1 << BlockingBits;
	} while (n -= 1 << BlockingBits);
}

#endif
