/*
Copyright (C) 2015 Alexey Karyakin <alexey.karyakin@uwaterloo.ca>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the "Software"), to deal in the Software without
restriction, including without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#include <stdint.h>
#include <iostream>
#include <queue>
#include <fstream>
#include <fcntl.h>

#include <stdio.h>
#include "util.h"
#include "PageStore.h"
#include "Reader.h"

PageStore g_page_store;

extern void processInput(Reader &in, uint64_t len, std::queue<char> *result = NULL);
extern void terminate();

int main(int argc, char *argv[])
{
	if (argc == 2)
	{
		//Handle input from file
		int ifs = open(argv[1], O_RDONLY);
		if (ifs < 0)
		{
			std::cerr << "read error" << std::endl;
			abort();
		}
		Reader in(ifs, 35, &g_page_store); // CONST 32 GB
		uint64_t pos = 0, val = 0;
		std::queue<char> result;
		while (true)
		{
			//Read length and process input
			auto *plen = in.read<uint32_t>();
			if (plen == NULL)
				break;
			auto len = *plen;
			processInput(in, len, &result);

			//Skip result reference data
			auto vlen = *in.read<uint32_t>();
			const char *correct = in.read<char>(vlen);

			if (!result.empty()) // for printer
			{
				for (uint32_t index = 0; index != vlen; ++index)
				{
					if (result.empty())
					{
						std::cout << "incomplete validation result "
								<< (index + val) << std::endl;
						abort();
					}
					if (result.front() != correct[index])
					{
						std::cout << "wrong validation result for validation "
								<< (index + val)
								<< ", got " << result.front()
								<< ", expected " << correct[index] << std::endl;
					}
					result.pop();
				}
			}
			pos += len;
			val += vlen;
		}
	}
	else
	{
		Reader in(0, 35, &g_page_store); // CONST 32 GB
		processInput(in, -1l);
	}
	terminate();
}
