/*
Copyright (C) 2015 Alexey Karyakin <alexey.karyakin@uwaterloo.ca>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the "Software"), to deal in the Software without
restriction, including without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#include "printer.h"
#include "protocol.h"
#include "util.h"
#include "Reader.h"

#include <vector>
#include <iostream>

static std::vector<unsigned> schema;

void printDefineSchema(const DefineSchema& d)
{
	std::cout << "defineschema [";
	for (uint32_t i = 0; i != d.relationCount; ++i)
	{
		schema.push_back(d.columnCounts[i]);
		std::cout << d.columnCounts[i] << " ";
	}
	std::cout << "]" << std::endl;
}

void printTransaction(Reader &in, const Transaction& t)
{
	for (uint32_t index = 0; index != t.deleteCount; ++index)
	{
		auto o = *in.read<TransactionOperationDelete>();
		for (uint32_t j = 0; j != o.rowCount; ++j)
		{
			auto v = *in.read<uint64_t>();
			std::cout << "D\t" << t.transactionId << "\t" << o.relationId << "\t" << v << std::endl;
		}
	}

	for (uint32_t index = 0; index != t.insertCount; ++index)
	{
		auto o = *in.read<TransactionOperationInsert>();
		for (uint32_t j = 0; j != o.rowCount; ++j)
		{
			std::cout << "I\t" << t.transactionId << "\t" << o.relationId;
			for (uint32_t i = 0; i != schema[o.relationId]; ++i)
			{
				auto v = *in.read<uint64_t>();
				std::cout << "\t" << v;
			}
			std::cout << std::endl;
		}
	}
}

void printQuery(const Query& q)
{
	std::cout << "[ " << q.relationId << " ";
	for (auto c = q.columns, cLimit = c + q.columnCount; c != cLimit; ++c)
	{
		std::cout << "c" << c->column;
		switch (c->op)
		{
		case Query::Column::Equal:
			std::cout << "=";
			break;
		case Query::Column::NotEqual:
			std::cout << "!=";
			break;
		case Query::Column::Less:
			std::cout << "<";
			break;
		case Query::Column::LessOrEqual:
			std::cout << "<=";
			break;
		case Query::Column::Greater:
			std::cout << ">";
			break;
		case Query::Column::GreaterOrEqual:
			std::cout << ">=";
			break;
		}
		std::cout << c->value << " ";
	}
	std::cout << "] ";
}

void printValidationQueries(const ValidationQueries& v)
{
	std::cout << "validation " << v.validationId << " " << v.from << " " << v.to << " ";
	const char *p = v.queries;
	for (unsigned index = 0; index != v.queryCount; ++index)
	{
		auto &q = *reinterpret_cast<const Query*>(p);
		printQuery(q);
		p += sizeof(Query) + sizeof(Query::Column) * q.columnCount;
	}
	std::cout << std::endl;
}

void printFlush(const Flush& f)
{
	std::cout << "flush " << f.validationId << std::endl;
}

void printForget(const Forget& f)
{
	std::cout << "forget " << f.transactionId << std::endl;
}
