/*
Copyright (C) 2015 Alexey Karyakin <alexey.karyakin@uwaterloo.ca>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the "Software"), to deal in the Software without
restriction, including without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#include "def.h"
#include "printer.h"
#include "protocol.h"
#include "util.h"
#include "Reader.h"

#include <stdlib.h>
#include <assert.h>
#include <queue>

static std::vector<uint32_t> schema;

static void processDefineSchema(const DefineSchema& d)
{
	schema.clear();
	schema.insert(schema.begin(), d.columnCounts, d.columnCounts + d.relationCount);
}

void processInput(Reader &in, uint64_t len, std::queue<char> * = NULL)
{
	uint64_t start = in.count();
	while (in.count() < start + len)
	{
		auto head = *in.read<MessageHead>();
		uint64_t next = in.count() + head.messageLen;
		switch (head.type)
		{
		case MessageHead::Done:
			return;
		case MessageHead::DefineSchema:
		{
			const auto& t = *in.read<DefineSchema>(head.messageLen);
			processDefineSchema(t);
			printDefineSchema(t);
			break;
		}
		case MessageHead::Transaction:
		{
			auto t = *in.read<Transaction>();
			printTransaction(in, t);
			break;
		}
		case MessageHead::ValidationQueries:
		{
			const auto &v = *in.read<ValidationQueries>(head.messageLen);
			printValidationQueries(v);
			break;
		}
		case MessageHead::Flush:
			printFlush(*in.read<Flush>());
			break;
		case MessageHead::Forget:
			printForget(*in.read<Forget>());
			break;
		default:
			fprintf(stderr, "malformed message" "\n");
			abort();
		}
		assert(next == in.count());
		(void) next;
	}
}

void terminate()
{
}
