/*
Copyright (C) 2015 Alexey Karyakin <alexey.karyakin@uwaterloo.ca>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the "Software"), to deal in the Software without
restriction, including without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef SRC_PROTOCOL_H_
#define SRC_PROTOCOL_H_

#include <cstdint>

//---------------------------------------------------------------------------
// Wire protocol messages
//---------------------------------------------------------------------------
struct MessageHead {
   /// Message types
   enum Type : uint32_t { Done, DefineSchema, Transaction, ValidationQueries, Flush, Forget };
   /// Total message length, excluding this head
   uint32_t messageLen;
   /// The message type
   Type type;
};
//---------------------------------------------------------------------------
struct DefineSchema {
   /// Number of relations
   uint32_t relationCount;
   /// Column counts per relation, one count per relation. The first column is always the primary key
   uint32_t columnCounts[];
};
//---------------------------------------------------------------------------
struct Transaction {
   /// The transaction id. Monotonic increasing
   uint64_t transactionId;
   /// The operation counts
   uint32_t deleteCount,insertCount;
   /// A sequence of transaction operations. Deletes first, total deleteCount+insertCount operations
//   char operations[];
};
//---------------------------------------------------------------------------
struct TransactionOperationDelete {
   /// The affected relation
   uint32_t relationId;
   /// The row count
   uint32_t rowCount;
   /// The deleted values, rowCount primary keys
//   uint64_t keys[];
};
//---------------------------------------------------------------------------
struct TransactionOperationInsert {
   /// The affected relation
   uint32_t relationId;
   /// The row count
   uint32_t rowCount;
   /// The inserted values, rowCount*relation[relationId].columnCount values
   uint64_t values[];
};
//---------------------------------------------------------------------------
struct Query {
   /// A column description
   struct Column {
      /// Support operations
      enum Op : uint32_t { Equal, NotEqual, Less, LessOrEqual, Greater, GreaterOrEqual };
      /// The column id
      uint32_t column;
      /// The operations
      Op op;
      /// The constant
      uint64_t value;
   };

   /// The relation
   uint32_t relationId;
   /// The number of bound columns
   uint32_t columnCount;
   /// The bindings
   Column columns[];
};
//---------------------------------------------------------------------------
struct ValidationQueries {
   /// The validation id. Monotonic increasing
   uint64_t validationId;
   /// The transaction range
   uint64_t from,to;
   /// The query count
   uint32_t queryCount;
   /// The queries
   char queries[];
};
//---------------------------------------------------------------------------
struct Flush {
   /// All validations to this id (including) must be answered
   uint64_t validationId;
};
//---------------------------------------------------------------------------
struct Forget {
   /// Transactions older than that (including) will not be tested for
   uint64_t transactionId;
};

#endif
